from typing import Any
from typing import List

from .base import VariableTracker
from .base import VariableTracker as Variable
from .builtin import BuiltinVariable
from .constant import ConstantVariable
from .dicts import ConstDictVariable
from .dicts import DataClassVariable
from .functions import NestedUserFunctionVariable
from .functions import UserFunctionVariable
from .functions import UserMethodVariable
from .lists import BaseListVariable
from .lists import ListIteratorVariable
from .lists import ListVariable
from .lists import NamedTupleVariable
from .lists import RangeVariable
from .lists import SliceVariable
from .lists import TupleVariable
from .misc import AutogradFunctionVariable
from .misc import BlackHoleVariable
from .misc import ClosureVariable
from .misc import ContextManagerVariable
from .misc import GetAttrVariable
from .misc import GradModeVariable
from .misc import InspectSignatureVariable
from .misc import LambdaVariable
from .misc import NewCellVariable
from .misc import NewGlobalVariable
from .misc import NumpyVariable
from .misc import PythonModuleVariable
from .misc import SuperVariable
from .misc import UnknownVariable
from .misc import WithExitFunctionVariable
from .nn_module import NNModuleVariable
from .nn_module import UnspecializedNNModuleVariable
from .tensor import TensorVariable
from .tensor import UnspecializedNumpyVariable
from .tensor import UnspecializedPythonVariable
from .torch import TorchVariable
from .user_defined import UserDefinedClassVariable
from .user_defined import UserDefinedObjectVariable


def is_literal(obj: Any) -> bool:
    if type(obj) in (int, float, bool, type(None), str):
        return True
    if type(obj) in (list, tuple, set, frozenset):
        return all(is_literal(x) for x in obj)
    return False


def propagate(*vars: List[List[Variable]]):
    return Variable.propagate(*vars)


def build(*args, **kwargs):
    from .builder import VariableBuilder

    return VariableBuilder(*args, **kwargs)


def constant(*args, **kwargs):
    return ConstantVariable(*args, **kwargs)


def baselist(*args, **kwargs):
    return ListVariable(*args, **kwargs)


def basetuple(*args, **kwargs):
    return TupleVariable(*args, **kwargs)


def listiter(*args, **kwargs):
    return ListIteratorVariable(*args, **kwargs)


def constdict(*args, **kwargs):
    return ConstDictVariable(*args, **kwargs)


def userfunc(*args, **kwargs):
    return UserFunctionVariable(*args, **kwargs)


def usermethod(*args, **kwargs):
    return UserMethodVariable(*args, **kwargs)


def torch(*args, **kwargs):
    return TorchVariable(*args, **kwargs)


__all__ = [
    "AutogradFunctionVariable",
    "BaseListVariable",
    "BlackHoleVariable",
    "BuiltinVariable",
    "ClosureVariable",
    "ConstantVariable",
    "ConstDictVariable",
    "ContextManagerVariable",
    "DataClassVariable",
    "GetAttrVariable",
    "GradModeVariable",
    "InspectSignatureVariable",
    "LambdaVariable",
    "ListIteratorVariable",
    "ListVariable",
    "NamedTupleVariable",
    "NestedUserFunctionVariable",
    "NewCellVariable",
    "NewGlobalVariable",
    "NNModuleVariable",
    "NumpyVariable",
    "PythonModuleVariable",
    "RangeVariable",
    "SliceVariable",
    "SuperVariable",
    "TensorVariable",
    "TorchVariable",
    "TupleVariable",
    "UnknownVariable",
    "UnspecializedNNModuleVariable",
    "UnspecializedNumpyVariable",
    "UnspecializedPythonVariable",
    "UserDefinedClassVariable",
    "UserDefinedObjectVariable",
    "UserFunctionVariable",
    "UserMethodVariable",
    "Variable",
    "VariableTracker",
    "WithExitFunctionVariable",
]
